const {
    types,
    operate,
} = require('putout');

const {isFunction, isIdentifier} = types;
const {compareAny} = operate;

const getKey = (a) => a.node.key.name;
module.exports.report = ({path, propsToRemove}) => {
    const {name} = path.node.callee;
    const names = propsToRemove.map(getKey);
    const message = `${name} passes useless argument props: ${names.join(', ')}`;
    
    return message;
};

module.exports.fix = ({path, propsToRemove}) => {
    for (const prop of propsToRemove) {
        prop.remove();
    }
};

module.exports.exclude = () => [
    '__.__()',
];

module.exports.traverse = ({push}) => {
    return {
        '__({})'(path) {
            const {name} = path.node.callee;
            const [argument] = path.get('arguments');
            const argProps = argument.get('properties');

            const binding = path.scope.bindings[name];
            const [param] = binding.path.get('params');

            if (!param.isObjectPattern())
                return;

            const {properties} = param.node;
            
            for (const prop of properties) {
                if (!isIdentifier(prop.value))
                    return;
            }
            const propsToRemove = [];
            for (const propPath of argProps) {
                const is = compareAny(propPath.node, properties);

                if (!is)
                    propsToRemove.push(propPath);
            }
            
            if (!propsToRemove.length)
                return;
            
            push({
                  path,
                propsToRemove,
            })
        }
    }
}

